/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import java.net.URI;
import java.util.concurrent.CompletableFuture;

import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.ozgcloud.antragsraum.common.InvalidFileTypeException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.headers.Header;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.ExampleObject;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(FileController.PATH)
@RequiredArgsConstructor
@SecurityRequirement(name = "security_auth")
public class FileController {
	public static final String PATH = "/api/file/";
	static final String PATH_PATTERN = PATH + "%s/%s";

	private final @NonNull FileService fileService;
	private final @NonNull FileValidator fileValidator;

	@Operation(summary = "Download file content")
	@GetMapping(value = "{nachrichtEventId}/{fileId}", produces = { MediaType.APPLICATION_OCTET_STREAM_VALUE, "images/*" })
	public ResponseEntity<StreamingResponseBody> getFileData(
	  @Parameter(description = "The id of the NachrichtEvent in the InfoManager the file belongs to", example = "60af924b4f1a2560298b4567")
	  @PathVariable String nachrichtEventId,
	  @Parameter(description = "The id of the file", example = "6358fd0bee7a051389cdd788") @PathVariable String fileId) {
		var ozgFile = getFile(fileId, nachrichtEventId);

		return buildResponseEntity(ozgFile, createDownloadStreamingBody(fileId, nachrichtEventId));
	}

	OzgFile getFile(String fileId, String nachrichtEventId) {
		return fileService.getFile(fileId, nachrichtEventId);
	}

	private ResponseEntity<StreamingResponseBody> buildResponseEntity(OzgFile ozgFile, StreamingResponseBody responseBody) {
		if (ozgFile != null) {
			return ResponseEntity.ok()
			  .header(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=%s", ozgFile.fileName()))
			  .contentLength(ozgFile.fileSize())
			  .contentType(MediaType.valueOf(ozgFile.contentType()))
			  .body(responseBody);
		} else {
			return ResponseEntity.notFound().build();
		}
	}

	private StreamingResponseBody createDownloadStreamingBody(String fileId, String nachrichtEventId) {
		return out -> fileService.downloadFileContent(fileId, out, nachrichtEventId);
	}

	@Operation(summary = "Upload file content",
	  description = "You can use cURL to upload a file example: curl -F file=@TestDokument.pdf http://localhost:8080/api/file/[vorgangid]/[nachrichtEventId] -v")
	@ApiResponses(value = {
	  @ApiResponse(responseCode = "201", description = "The file has been created",
		content = @Content(examples = @ExampleObject(value = "6358fd0bee7a051389cdd788", description = "The id of the file created")),
		headers = { @Header(name = "Location", description = "The path of of the uploaded file with its id") })
	})
	@PostMapping(value = "{vorgangId}/{nachrichtEventId}", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	@Async
	public CompletableFuture<ResponseEntity<String>> uploadFile(
	  @Parameter(description = "The id of the Vorgang the clerk message belongs to", example = "6358fd0bee7a051389cdd787")
	  @PathVariable @NonNull String vorgangId,
	  @Parameter(description = "The id of the NachrichtEvent in the InfoManager the file belongs to", example = "60af924b4f1a2560298b4567")
	  @PathVariable @NonNull String nachrichtEventId,
	  @RequestBody MultipartFile file) {
		LOG.info("Uploading attachment for vorgang {} for nachricht event id {}", vorgangId, nachrichtEventId);

		if (fileValidator.isValid(file)) {
			var fileIdFuture = fileService.upload(vorgangId, nachrichtEventId, file);

			return fileIdFuture.thenApply(fileId -> ResponseEntity.created(URI.create(PATH_PATTERN.formatted(vorgangId, fileId))).body(fileId));
		} else {
			LOG.warn("Uploaded File {} is not valid.", file.getName());
			throw new InvalidFileTypeException(file.getName());
		}
	}
}
