/*
 * Copyright (c) 2024.   Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.WebConfiguration.*;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

import org.apache.commons.io.IOUtils;
import org.springframework.core.io.Resource;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestClient;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.common.RestClientUtils;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcFindFilesResponse;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
@RequiredArgsConstructor
public class FileRestClient {
	private static final String FIND_FILE_METADATA_API = "/api/v1/file/metadata/{fileId}";
	private static final String GET_FILE_CONTENT_API = "/api/v1/file/content/{fileId}";
	static final String UPLOAD_FILE_API = "/api/v1/file";
	private final @NonNull ContextService contextService;
	private final @NonNull RestClient nachrichtRestClient;

	public AntragraumproxyGrpcFindFilesResponse findBinaryFilesMetaData(String fileId, final String address) {
		return nachrichtRestClient
		  .get()
		  .uri(FIND_FILE_METADATA_API, fileId)
		  .accept(MediaType.APPLICATION_JSON)
		  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
		  .retrieve()
		  .toEntity(AntragraumproxyGrpcFindFilesResponse.class).getBody();
	}

	public void uploadFile(final OzgUploadFile uploadFile, final String address, final CompletableFuture<String> fileIdFuture) {
		try {
			var response = sendFile(uploadFile, address);
			if (Objects.nonNull(response)) {
				fileIdFuture.complete(response);
			} else {
				fileIdFuture.completeExceptionally(new IllegalStateException("No fileId received"));
			}
		} catch (Exception e) {
			fileIdFuture.completeExceptionally(e);
		}
	}

	private String sendFile(OzgUploadFile uploadFile, String address) {

		MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();
		requestBody.add("file", uploadFile.file().getResource());
		requestBody.add("vorgangId", uploadFile.vorgangId());

		var fileId = nachrichtRestClient
		  .post()
		  .uri(UPLOAD_FILE_API)
		  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
		  .contentType(MediaType.MULTIPART_FORM_DATA)
		  .body(requestBody)
		  .retrieve().toEntity(String.class).getBody();
		LOG.info("received {}", fileId);

		return fileId;
	}

	public void downloadFileContent(final String fileId, final OutputStream out, final String address) throws IOException {
		var fileContent = Optional.ofNullable(nachrichtRestClient
		  .get()
		  .uri(GET_FILE_CONTENT_API, fileId)
		  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
		  .accept(MediaType.APPLICATION_OCTET_STREAM)
		  .retrieve().toEntity(Resource.class).getBody());

		fileContent.ifPresent(file -> processFile(out, file));
	}

	private static void processFile(final OutputStream out, Resource fileContent) {
		try {
			IOUtils.copy(fileContent.getInputStream(), out);
		} catch (IOException e) {
			LOG.error("Error writing file content to output", e);
			throw new TechnicalException(e);
		}
	}
}
