/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.WebConfiguration.*;
import static de.ozgcloud.antragsraum.attachments.FileRestClient.*;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Function;

import org.apache.commons.io.IOUtils;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestClient;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcUploadBinaryFileResponse;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@AllArgsConstructor
class RestChunkedFileSender<T> {
	private final AtomicBoolean hasUploadFile = new AtomicBoolean(true);
	private final InputStream uploadStream;
	private final int chunkSize;
	private final Function<byte[], T> buildChunkRequest;
	private T requestMetadata;
	private final RestClient restClient;
	private String targetAddress;
	private final CompletableFuture<String> fileIdFuture;

	public void sendChunkTo() {
		if (hasUploadFile.get()) {
			var response = sendMetadata();
			int size = sendNextChunk();
			if (size < chunkSize) {
				handleFileEndReached();
				fileIdFuture.complete(response.getFileId());
			}
		}
	}

	private AntragraumproxyGrpcUploadBinaryFileResponse sendMetadata() {
		if (requestMetadata != null) {
			var res = restClient
			  .post()
			  .uri(UPLOAD_FILE_API)
			  .header(X_GRPC_ADDRESS, targetAddress)
			  .contentType(MediaType.APPLICATION_JSON)
			  .body(requestMetadata)
			  .retrieve().toEntity(AntragraumproxyGrpcUploadBinaryFileResponse.class).getBody();
			LOG.info("recieved {}", res);

			requestMetadata = null;

			return res;
		}

		return null;
	}

	private int sendNextChunk() {
		byte[] content = readFromStream();
		var size = content.length;
		if (size > 0) {
			var restUploadFile = buildChunkRequest.apply(content);
			var res = restClient
			  .post()
			  .uri(UPLOAD_FILE_API)
			  .header(X_GRPC_ADDRESS, targetAddress)
			  .contentType(MediaType.APPLICATION_JSON)
			  .body(restUploadFile)
			  .retrieve().toEntity(AntragraumproxyGrpcUploadBinaryFileResponse.class);

			LOG.info("Send binary chunk result: {}", res);
		}
		return size;
	}

	private byte[] readFromStream() {
		try {
			return uploadStream.readNBytes(chunkSize);
		} catch (IOException e) {
			LOG.error("Error on sending a single chunk", e);
			throw new TechnicalException(e);
		}
	}

	private void handleFileEndReached() {
		IOUtils.closeQuietly(uploadStream);

		hasUploadFile.getAndSet(false);
	}
}
