/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Collection;
import java.util.List;

import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.common.NotAccessibleException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class NachrichtenService {
	static final String RUECKFRAGE_LINK_RELATIONSHIP_NAME = "rueckfrage";

	private final @NonNull NachrichtEventService nachrichtEventService;
	private final @NonNull NachrichtenRemoteService nachrichtenRemoteService;

	List<RueckfrageHeader> getRueckfrageHeadersOfPostfach(String postfachId) {
		var headers = RueckfrageHeaderMapper.fromNachrichtHeaders(getNachrichtHeadersOfPostfach(postfachId));
		linkRueckfrageHeaders(headers);

		return headers;
	}

	Collection<NachrichtHeader> getNachrichtHeadersOfPostfach(String postfachId) {
		var events = nachrichtEventService.getNachrichtEventsOfPostfachId(postfachId);

		return events.stream().map(this::getNachrichtHeadersOfEvent).flatMap(List::stream).toList();
	}

	private List<NachrichtHeader> getNachrichtHeadersOfEvent(final NachrichtEvent event) {
		var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(event);
		LOG.debug("{} NachrichtHeaders für Postfach ID {} erhalten", nachrichtHeaders.size(), event.postfachId());

		return nachrichtHeaders;
	}

	private void linkRueckfrageHeaders(List<RueckfrageHeader> headers) {
		headers.forEach(header -> {
			header.add(linkTo(NachrichtenController.class).withSelfRel());

			var nachrichtHeader = header.getNachrichtHeader();
			if (nachrichtHeader.accessible()) {
				header.add(
				  linkTo(methodOn(NachrichtenController.class).getRueckfrage(nachrichtHeader.id(), nachrichtHeader.nachrichtEventId())).withRel(
					RUECKFRAGE_LINK_RELATIONSHIP_NAME));
			}
		});
	}

	Rueckfrage getRueckfrage(String rueckfrageId, String nachrichtEventId) {
		var nachrichtEvent = nachrichtEventService.getNachrichtEventById(nachrichtEventId);
		var nachricht = nachrichtenRemoteService.getRueckfrage(rueckfrageId, nachrichtEvent).orElseThrow();

		if (!nachricht.accessible()) {
			throw new NotAccessibleException(rueckfrageId);
		}

		return RueckfrageMapper.fromNachricht(nachricht);
	}

	CommandReference sendRueckfrageAnswer(final ReplyNachricht nachricht) {
		var nachrichtEvent = nachrichtEventService.getNachrichtEventById(nachricht.nachrichtEventId());

		return nachrichtenRemoteService.sendAnswer(nachricht, nachrichtEvent.address());
	}
}
