/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcFindFilesResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcOzgFile;

@ExtendWith(MockitoExtension.class)
class FileRemoteServiceTest {
	@Spy
	@InjectMocks
	private FileRemoteService remoteService;

	@Mock
	private FileRestClient restClient;

	@Nested
	class TestLoadingFile {

		@BeforeEach
		void init() {
			var response = new AntragraumproxyGrpcFindFilesResponse();
			var file = new AntragraumproxyGrpcOzgFile();
			file.setName(FILE_NAME);
			file.setSize(FILE_SIZE_STRING);
			file.setContentType(CONTENT_TYPE);
			file.setId(FILE_ID);
			response.setFile(List.of(file));
			when(restClient.findBinaryFilesMetaData(any(), any())).thenReturn(response);
		}

		@Test
		void shouldLoadFileMetadata() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file).isNotNull();
		}

		@Test
		void shouldHaveFileName() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.fileName()).isEqualTo(FILE_NAME);
		}

		@Test
		void shouldHaveFileSize() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.fileSize()).isEqualTo(FILE_SIZE);
		}

		@Test
		void shouldHaveContentType() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.contentType()).isEqualTo(CONTENT_TYPE);
		}

		@Test
		void shouldHaveFileId() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.id()).isEqualTo(FILE_ID);
		}

		@Test
		void shouldHandleEmpty() {
			var response = new AntragraumproxyGrpcFindFilesResponse();
			when(restClient.findBinaryFilesMetaData(any(), any())).thenReturn(response);

			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file).isNull();
		}
	}

	@Nested
	class TestDownloadFileContent {

		private final ByteArrayOutputStream output = new ByteArrayOutputStream();

		@Test
		void shouldCallDownloadFileObserver() throws Exception {
			remoteService.downloadFileContent(FILE_ID, output, CHANNEL_ADDRESS);

			verify(restClient).downloadFileContent(eq(FILE_ID), any(OutputStream.class), eq(CHANNEL_ADDRESS));
		}
	}

	@Nested
	class TestUploadFile {

		@Test
		void shouldCallRestClient() {
			remoteService.uploadFile(OzgUploadFileTestFactory.create(), CHANNEL_ADDRESS);

			verify(restClient).uploadFile(any(OzgUploadFile.class), eq(CHANNEL_ADDRESS), any());
		}
	}
}