/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatusCode;

import java.util.Optional;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class AuthenticationControllerTest {
    @Spy
    @InjectMocks
    private AuthenticationController authenticationController;

    @Mock
    private InMemoryUserDetailService inMemoryUserDetailService;

    @Mock
    private JwtTokenProvider tokenProvider;

    @Nested
    class TestLogin {
        private AuthCode code;

        @BeforeEach
        void setup() {
            code = new AuthCode(RandomStringUtils.randomAlphanumeric(20));
        }

        @Test
        void shouldCreateTokenResponse() {
            var tokenResponse = authenticationController.login(code);

            assertThat(tokenResponse).isNotNull();
        }

        @Test
        void shouldHaveJwtToken() {
            when(inMemoryUserDetailService.getUserByCode(anyString())).thenReturn(UserTestFactory.create());
            when(tokenProvider.generate(any(User.class))).thenReturn("test");

            var tokenResponse = authenticationController.login(code);

            assertThat(tokenResponse.getBody()).isNotNull();
        }

        @Test
        void shouldHaveStatusOk() {
            var tokenResponse = authenticationController.login(code);

            assertThat(tokenResponse.getStatusCode()).isEqualTo(HttpStatusCode.valueOf(200));
        }

        @Test
        void shouldCallTokenProvider() {
            when(inMemoryUserDetailService.getUserByCode(anyString())).thenReturn(UserTestFactory.create());

            authenticationController.login(code);

            verify(tokenProvider).generate(any(User.class));
        }

        @Test
        void shouldCallUSerService() {
            authenticationController.login(code);

            verify(inMemoryUserDetailService).getUserByCode(anyString());
        }

        @Test
        void shouldThrowSecurityException() {
            when(tokenProvider.generate(any())).thenThrow(IllegalArgumentException.class);

            assertThatExceptionOfType(SecurityException.class).isThrownBy(() -> authenticationController.login(code));
        }

    }

    @Nested
    class TestRefresh {
        private AuthCode code;

        @BeforeEach
        void setup() {
            code = new AuthCode(RandomStringUtils.randomAlphanumeric(20));
        }

        @Test
        void shouldCreateToken() {
            when(inMemoryUserDetailService.getUser(eq(code))).thenReturn(Optional.of(UserTestFactory.create()));

            var token = authenticationController.refresh(code);

            assertThat(token.getStatusCode()).isEqualTo(HttpStatusCode.valueOf(200));
        }

        @Test
        void shouldNotCreateToken() {
            var token = authenticationController.refresh(code);

            assertThat(token.getStatusCode()).isEqualTo(HttpStatusCode.valueOf(403));
        }

        @Test
        void shouldCallUserDetailServiceUpdate() {
            when(inMemoryUserDetailService.getUser(eq(code))).thenReturn(Optional.of(UserTestFactory.create()));
            
            authenticationController.refresh(code);

            verify(inMemoryUserDetailService).updateRefreshCodeOf(any(User.class));
        }
    }
}